package com.zayhu;

import android.os.Bundle;
import com.zayhu.IZayhuServiceCallback;
import com.zayhu.IZayhuServiceProbeCallback;

/**
 */
interface IZayhuServiceConnection {
    void setCallback(IZayhuServiceCallback cb);

    // once login, service will keep online when connected
    // loginEntry is serialized to byte[] to transfer by binder
    boolean login(long rid, in byte[] loginEntry);

    // when logged out, service is disabled
    boolean logout(long rid);

    boolean relogin(long rid);
    boolean forceRegister(long oldRID, long newRID, String statusText);

    // if login info present && network connected, wait to push channel ready with packed login info;
    // otherwise return null immediately
    String waitForLogin();

    // same as waitForLoginInfo except no wait
    String getLoginNoWait();

    long getAcitveRID();

    String getIdcConfig();
    String getShadowConfig();
    boolean applyShadowConfig(boolean enabled, String configString, String configUrl);

    boolean isPushReady();

    // 0        Fully functional, seems OK
    //
    // 1 << 0   Push could not be connected, nothing working
    // 1 << 1   UDP transport not available, voice quality will degraded
    // 1 << 2   be killed too many times, device out of memory
    // 1 << 3   could not be recovered after be killed
    int getRecentServiceLevel();
    void setRecentServiceLevel(int flag, boolean effective);

    void markBadPushServer(String pushServer);
    void clearBadPushServer();

    boolean probeIdc(long probeSessionId, int probeType, IZayhuServiceProbeCallback probeCB, int countryCode, String hid);

    void reportPushProtocolError();

    byte[] getCertificate(String fingerprint);
    String[] getCertificateFingerprints();

    // For test. return current apply udp & proxy & tcp server address info
    String getCurrentServers();
    boolean forceApplyServers(String serverJson);

    void sendMissedCallStat(String json);

    int setConfig(int cFlag, int value, String extra);
    int getConfigValue(int cFlag);
    String getConfigExtra(int cFlag);
}