package com.zayhu;

import com.zayhu.IYDataPacketCallback;
import com.zayhu.IYPushPacketCallback;
import com.zayhu.IYPushPacketSendResult;
import com.zayhu.IZayhuSessionMonitor;
import com.zayhu.IYVideoPacketCallback;

interface IZayhuVoIP {
    /**
     * make a phone call to peer
     *
     * @param peerHID: peer hid to be called
     * @param appid: application identifier
     * @return status code for this action
     */
    int call(String peerHID, int appId, String extraJsonObject);

    /**
     * Accept the incoming call
     */
    int acceptCall(String peerHID, int appId, boolean handsFree);

    /**
     * End current call
     * 1. hangUp: terminate phone call gracefully
     * 2. reject: callee initiated event: phone call is not established
     * 3. cancel: caller initiated event; phone call is not established
     */
    int endCall(int endType);

    /**
     * Get state machine state now
     */
    int currentState();

    /**
     * Get current mode flags mask
     */
    long getMode();

    /**
     * Set several mode flags
     * 1. mute mic
     * 2. mute mic & speaker
     * 3. hands free mode
     * 4. push to talk mode
     */
    int setMode(long modeFlag, boolean enable);

    /**
     * Start ptt recording for maxium 'maxMillis' ms
     */
    int pttStart(long maxMillis);

    /**
     * Stop last ptt recording; ptt recording might terminate early since maxMillis provided
     */
    int pttStop();

    /**
     * add / remove callbacks for voice session state
     */
    void setSessionMonitor(IZayhuSessionMonitor callback);
    void clearSessionMonitor();

    // ------------ BEGIN Data Packet Processing --------------------
    void addDataPacketCallback(int appId, int type, IYDataPacketCallback callback);
    void removeDataPacketCallback(int appId, int type);

    boolean sendDatagramPacketToActivePeer(int appId, int type, int priority, in byte[] payload);
    // ------------ END Data Packet Processing --------------------

    // ------------ BEGIN Push Packet Processing --------------------
    void addPushPacketCallback(String category, IYPushPacketCallback callback);
    void removePushPacketCallback(String category);

    boolean sendPushPacketToActivePeer(String category, int appId, int priority, in String message, in byte[] extras);

    boolean sendAppTransferToActivePeer(String category, int appId, int priority, in String message, in byte[] extras);

    boolean sendPushPacket(int priority, in String message, in byte[] extras);
    boolean sendPushPacketWithResult(int priority, in String message, in byte[] extras, String sendTag, IYPushPacketSendResult sendResult);

    boolean removePushPacket(String category, String uuid, String hid);
    // ------------ END Push Packet Processing --------------------

    // ------------ BEGIN Video Related Processing --------------------
    /**
     *  Start / Stop Video Call
     */
    boolean startVideoCall(in IYVideoPacketCallback callback);
    void stopVideoCall();
    void sendVideoFrameToActivePeer(int appId, int type, int priority, in byte[] videoFrameByte, int offset, int length);
    void askForIFrame(int nackType, in String nackJsonString);
    int isPeerSupportVideo();
    int isVideoCallSupported();
    String getDeviceConfigJson();
    String getVideoConfigJson();
    void forceOpenVideoFeature(boolean useVideoCall);
    boolean isForceOpenVideo();
    String getCallOptions();
    void addVideoCallUser(String hid);
    void removeVideoCallUser(String hid);
    void startNetworkStatRightNow();
    float getNetworkRoundTripTime();
    boolean isUDPProxyByServer();
    // ------------ END Video Related Processing --------------------

    void switchToApp(int appId);
    int currentAppId();
    int currentRequestAppId();
    String currentPeer();
    String currentSessionId();
    String currentSender();
    boolean isInvitor();

    void stopRinging();

    int setRecorderConfig(int configID, int value);
    int getRecorderConfig(int configID);

    int getLastRecordingNormalize();
    void setupData(String jsonObjData);

    int setAction(int actionID, int value, String extras);
    int getActionValue(int actionID);
    String getActionExtras(int actionID);
}
