precision mediump float;

varying vec2 v_texcoord;
uniform sampler2D s_texture_y;
uniform sampler2D s_texture_u;
uniform sampler2D s_texture_v;
uniform float animatedProgress;
uniform float isMirroredH;

void main()
{
    vec3 centralColor;
    vec3 sampleColor;
    vec3 sum = vec3(0.0, 0.0, 0.0);
    float sumGaussian = 0.0;
    float distanceFromCentralColor;
    float gaussian[9];
    float gaussianWeight;
    int   distanceFromCentralCoord;
    
    gaussian[0] = 0.10;
    gaussian[1] = 0.15;
    gaussian[2] = 0.12;
    gaussian[3] = 0.09;
    gaussian[4] = 0.05;
    //     gaussian[5] = 0.02;
    //     gaussian[6] = 0.008;
    //     gaussian[7] = 0.002;
    //     gaussian[8] = 0.0005;

    float y = texture2D(s_texture_y, v_texcoord).r;
    float u = texture2D(s_texture_u, v_texcoord).r - 0.5;
    float v = texture2D(s_texture_v, v_texcoord).r - 0.5;
    centralColor = vec3(y, u, v);

    for ( int i = -4; i<=4; i++ ){
        int j = 0;
        vec2 gcoord = v_texcoord + vec2(float(i)*1.0/240.0, float(j)*1.0/240.0);
        float y = texture2D(s_texture_y, gcoord).r;
        float u = texture2D(s_texture_u, gcoord).r - 0.5;
        float v = texture2D(s_texture_v, gcoord).r - 0.5;
        sampleColor = vec3(y, u, v);
        distanceFromCentralColor = min(distance(sampleColor, centralColor) * 8.0, 1.04);
        distanceFromCentralCoord = int(abs(float(i))); //int(distance(vec2(float(i),float(j)), vec2(0.0,0.0)));
        gaussianWeight = gaussian[distanceFromCentralCoord] * (1.0 - distanceFromCentralColor);
        sum += vec3(y,u,v) * gaussianWeight;
        sumGaussian += gaussianWeight;
    }

    for ( int j = -4; j<=4; j++){
        int i = 0;
        vec2 gcoord = v_texcoord + vec2(float(i)*1.0/320.0, float(j)*1.0/320.0);
        float y = texture2D(s_texture_y, gcoord).r;
        float u = texture2D(s_texture_u, gcoord).r - 0.5;
        float v = texture2D(s_texture_v, gcoord).r - 0.5;
        sampleColor = vec3(y, u, v);
        distanceFromCentralColor = min(distance(sampleColor, centralColor) * 8.0, 1.04);
        distanceFromCentralCoord = int(abs(float(j)));// int(distance(vec2(float(i),float(j)), vec2(0.0,0.0)));
        gaussianWeight = gaussian[distanceFromCentralCoord] * (1.0 - distanceFromCentralColor);
        sum += vec3(y,u,v) * gaussianWeight;
        sumGaussian += gaussianWeight;
    }

    sum = sum/sumGaussian;
    float aRatio;
    float distanceFromProgress = (1.0 - 2.0*isMirroredH) * v_texcoord[1] + isMirroredH - animatedProgress;;
    if ( distanceFromProgress < 0.0 ){
        aRatio = 1.0;
    }
    else{
        aRatio = 0.0;
    }
    sum = mix(centralColor, sum, aRatio);
    sum[0] = (0.05 * aRatio + 1.0) * sum[0];

    float rr = sum[0] +                  1.402 * sum[2];
    float gg = sum[0] - 0.344 * sum[1] - 0.714 * sum[2];
    float bb = sum[0] + 1.772 * sum[1];
    //     gl_FragColor = vec4(rr, gg, bb, 1.0);

    //     float rgbSum = rr + gg + bb;
    //     float alpha_r = rr / rgbSum;
    //     float alpha_g = gg / rgbSum;
    //     float alpha_b = bb / rgbSum;
    //     float alpha = max(max(alpha_r, alpha_g), alpha_b);
    //     float minalpha = min(min(alpha_r, alpha_b), alpha_b);
    //     float MaxC = max(max(rr,gg), bb);
    //     float beta_r = (alpha_r - minalpha)/(1.0 - 3.0*minalpha);
    //     float beta_g = (alpha_g - minalpha)/(1.0 - 3.0*minalpha);
    //     float beta_b = (alpha_b - minalpha)/(1.0 - 3.0*minalpha);
    //     float beta = max(max(beta_r, beta_g), beta_b);
    //     float delta = (beta * rgbSum - MaxC)/(3.0*beta-1.0);
    //
    //     rr = rr - delta;
    //     gg = gg - delta;
    //     bb = bb - delta;

    gl_FragColor = vec4(rr, gg, bb, 1.0);
}